// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2
#ifndef DUNE_MATRIX_VECTOR_COMPONENTWISE_VECTOR_MAPS
#define DUNE_MATRIX_VECTOR_COMPONENTWISE_VECTOR_MAPS
#include <dune/common/hybridutilities.hh>
#include <dune/common/typetraits.hh>

namespace Dune {
namespace MatrixVector {

  /** Apply a function f componentwise to a vector v and store it in u, i.e.
   *  u_i = f(v_i)
   */
  template<typename F, typename V>
  void applyComponentwise(F&& f, V& u, const V& v) {
    namespace H = Dune::Hybrid;
    H::ifElse(Dune::IsNumber<std::decay_t<V>>(), [&](auto&& id) {
        u= f(id(v));},
        [&](auto id) {
        H::forEach(H::integralRange(H::size(id(v))), [&](auto i) {
            applyComponentwise(f,H::elementAt(u,i), H::elementAt(v,i));
            });
        });
  }

  /** Multiply the i-th component of u by the i-th component of v, i.e.
   *  u_i <- u_i*v_i
   */
  template<typename V>
  void multiplyComponentwise(V& u, const V& v) {
    namespace H = Dune::Hybrid;
    H::ifElse(Dune::IsNumber<std::decay_t<V>>(), [&](auto&& id) {
        u*=id(v);},
        [&](auto id) {
        H::forEach(H::integralRange(H::size(id(v))), [&](auto i) {
            multiplyComponentwise(H::elementAt(u,i), H::elementAt(v,i));
            });
        });
  }

}
}
#endif
